@@ -1,5 +1,4 @@
-from emBRICK.modbus_rtu import local
-from emBRICK.modbus_rtu import slave
+
 from emBRICK.modbus_rtu import bB_update
 from emBRICK.modbus_rtu import bB
 from emBRICK.modbus_rtu import connect
@@ -13,10 +12,8 @@ import scipy.interpolate
 
 lock = threading.Lock()
 
-
-
-MODE = ''
 brick_info = ''
+lock.acquire()
 def findBricks():
     '''
     Check connected Bricks and configure the corresponding objects
@@ -31,38 +28,38 @@ def findBricks():
     bricks = {2181: G8Di8Do, 4602: B3U4I, 4603: B3U4I, 2470: G2Mi2Ao, 2471: G2Mi2Ao, 2472: G2Mi2Ao, 2431: G4Ai4Tmp,
               2432: G4Ai4Tmp, 2433: G4Ai4Tmp, 2434: G4Ai4Tmp, 2435: G4Ai4Tmp, 2436: G4Ai4Tmp}
     '''
-    print(f'With following Bricks:')
     for node in connect.unit_id:
-        for i in range(local.info[node]['number of connected bricks']):
-            id = slave.id[node, i]
+        for i in range(connect.master["local" + str(node)].number_bricks):
+            id = connect.slave["master" + str(node)]["Brick" + str(i + 1)].brick_id
             found = bricks.get(id, "None")
             if found == "None":
                 error = open("error.txt", "a+")
                 error.write(f'{datetime.now()} ERROR: No Brick with ID {id} found.\n')
                 error.close()
-                brick_info += f'{i + 1}: ID: {slave.id[node, i] // 1000}-{slave.id[node, i] % 1000} Instance: None Hardware Revision: {slave.hard_rev[node, i]}\n'
+                brick_info += f'NOT FOUND! Brick {i+1}: {connect.slave["master" + str(node)]["Brick" + str(i+1)].name} '\
+                f'ID: {connect.slave["master" + str(node)]["Brick" + str(i+1)].brick_id // 1000}-{connect.slave["master" + str(node)]["Brick" + str(i+1)].brick_id % 1000} '\
+                f'Hardware Revision: {connect.slave["master" + str(node)]["Brick" + str(i+1)].hard_rev}\n'
             else:
-                globals()[f"Brick{i + 1}"] = found()
+                globals()[f"Brick{i + 1}"] = found(node, bricks[id], i+1, \
+                connect.slave["master" + str(node)]["Brick" + str(i+1)].brick_id,\
+                connect.slave["master" + str(node)]["Brick" + str(i+1)].data_miso,\
+                connect.slave["master" + str(node)]["Brick" + str(i+1)].data_mosi,\
+                connect.slave["master" + str(node)]["Brick" + str(i+1)].offset_miso,\
+                connect.slave["master" + str(node)]["Brick" + str(i+1)].offset_mosi)
 
                 found = "Brick"+str(i+1)
-                globals()[f"Brick{i+1}"].node = node
-                globals()[f"Brick{i+1}"].name = bricks[id]
-                globals()[f"Brick{i+1}"].brick_id = slave.id[node, i]
-                globals()[f"Brick{i+1}"].brick_no = i + 1
-                globals()[f"Brick{i+1}"].data_length_miso = slave.data_length_miso[node, i]
-                globals()[f"Brick{i+1}"].data_length_mosi = slave.data_length_mosi[node, i]
-                globals()[f"Brick{i+1}"].offset_miso = slave.offset_miso[node, i]
-                globals()[f"Brick{i+1}"].offset_mosi = slave.offset_mosi[node, i]
-                brick_info += f'{i + 1}: ID: {slave.id[node, i] // 1000}-{slave.id[node, i] % 1000} Instance: {found} Hardware Revision: {slave.hard_rev[node, i]}\n'
+
+                brick_info += f'{found}: {connect.slave["master" + str(node)]["Brick" + str(i+1)].name} '\
+                f'ID: {connect.slave["master" + str(node)]["Brick" + str(i+1)].brick_id // 1000}-{connect.slave["master" + str(node)]["Brick" + str(i+1)].brick_id % 1000} '\
+                f'Hardware Revision: {connect.slave["master" + str(node)]["Brick" + str(i+1)].hard_rev}\n'
     print(brick_info)
     print("To abort the program please press STRG + C\n\n")
-    global MODE
-    MODE = input("To Start press 'y' or to exit the application press 'e'")
-    while MODE != 'y' and MODE != 'e':
-        MODE = input("To Start press 'y' or to exit the application press 'e'")
-    if MODE == 'e':
-        bB_update.stopThread = True
-        exit()
+
+
+def start():
+    return lock.release()
+
+
 
 class CAE_G8Di8Do:
     '''
@@ -71,18 +68,18 @@ class CAE_G8Di8Do:
     digital_input(di_pos)
     digital_output(do_pos, value)
     '''
-    def __init__(self):
-        self.node = ''
-        self.name = ''
+    def __init__(self, node, name, brick_no, brick_id, data_length_miso, data_length_mosi, offset_miso, offset_mosi):
+        self.node = node
+        self.name = name
         self.id = 2181
-        self.brick_no = ''
-        self.brick_id = ''
+        self.brick_no = brick_no
+        self.brick_id = brick_id
         # Data Lenght
-        self.data_length_miso = ''
-        self.data_length_mosi = ''
+        self.data_length_miso = data_length_miso
+        self.data_length_mosi = data_length_mosi
         # Offset
-        self.offset_miso = ''
-        self.offset_mosi = ''
+        self.offset_miso = offset_miso
+        self.offset_mosi = offset_mosi
 
     def digital_input(self, di_pos):
         '''
@@ -130,9 +127,6 @@ class CAE_G8Di8Do:
         lock.release()
 
 
-G8Di8Do = CAE_G8Di8Do()
-
-
 class CAE_B3U4I:
     ''' Brick Information can find in eB_products.pdf unter Chapter 5.3.7 B-3U4I-400-##-RB
     Integrated Functions:
@@ -145,18 +139,18 @@ class CAE_B3U4I:
     readCurrent()
     readVoltage()
     '''
-    def __init__(self):
-        self.node = ''
-        self.name = ''
+    def __init__(self, node, name, brick_no, brick_id, data_length_miso, data_length_mosi, offset_miso, offset_mosi):
+        self.node = node
+        self.name = name
         self.id = [4602, 4603]
-        self.brick_id = ''
-        self.brick_no = ''
+        self.brick_id = brick_id
+        self.brick_no = brick_no
         # Data Lenght
-        self.data_length_miso = ''
-        self.data_length_mosi = ''
+        self.data_length_miso = data_length_miso
+        self.data_length_mosi = data_length_mosi
         # Offset
-        self.offset_miso = ''
-        self.offset_mosi = ''
+        self.offset_miso = offset_miso
+        self.offset_mosi = offset_mosi
         self.avg_interval = 10
         # PreConfigured Current, Voltage, Power Correction Variables 4-602
         self.cur_val_602 = 0.1873508544
@@ -176,6 +170,13 @@ class CAE_B3U4I:
         self.current = [0] * 4
         self.voltage = [0] * 3
         self.energy = [0] * 3
+        self.run = False
+
+    def start(self):
+        self.run = True
+    
+    def stop(self):
+        self.run = False
 
     def phase_cor(self, phase_cor_angle_l1, phase_cor_angle_l2, phase_cor_angle_l3):
         '''
@@ -330,7 +331,8 @@ class CAE_B3U4I:
             self.offset_miso + 8]
         power_l3 = (bB_update.updated[self.node][self.offset_miso + 9] << 8) + bB_update.updated[self.node][
             self.offset_miso + 10]
-        power = [power_l1, power_l2, power_l3]
+        power = [200, 100, 50]
+        #power = [power_l1, power_l2, power_l3]
         if self.brick_id == 4602:
             self.power = [i * self.pow_val_602 if i > self.filter_max or i < self.filter_min else 0 for i in np.int16(power)]
         else:
@@ -342,19 +344,20 @@ class CAE_B3U4I:
         '''
         Calculate and Return the Energy[L1, L2, L3] in Ws from Power[L1, L2, L3]
         '''
-        energy_val = self.avg_interval / 100
-        self.result_cnt = self.resultCnt()
-        if self.result_cnt != self.old_result_cnt:
-            self.old_result_cnt = self.result_cnt
-            self.energy[0] += energy_val * self.power[0]
-            if self.energy[0] > sys.maxsize:
-                self.energy[0] = 0
-            self.energy[1] += energy_val * self.power[1]
-            if self.energy[1] > sys.maxsize:
-                self.energy[1] = 0
-            self.energy[2] += energy_val * self.power[2]
-            if self.energy[2] > sys.maxsize:
-                self.energy[2] = 0
+        if self.run:
+            energy_val = self.avg_interval / 100
+            self.result_cnt = self.resultCnt()
+            if self.result_cnt != self.old_result_cnt:
+                self.old_result_cnt = self.result_cnt
+                self.energy[0] += energy_val * self.power[0]
+                if self.energy[0] > sys.maxsize:
+                    self.energy[0] = 0
+                self.energy[1] += energy_val * self.power[1]
+                if self.energy[1] > sys.maxsize:
+                    self.energy[1] = 0
+                self.energy[2] += energy_val * self.power[2]
+                if self.energy[2] > sys.maxsize:
+                    self.energy[2] = 0
         return self.energy
 
     def readPhase(self):
@@ -442,9 +445,6 @@ class CAE_B3U4I:
         return self.voltage
 
 
-B3U4I = CAE_B3U4I()
-
-
 class CAE_G2Mi2Ao:
     '''
     Brick Information can find in eB_products.pdf unter Chapter 1.6.11 G-2Mi2Ao-0#
@@ -466,18 +466,18 @@ class CAE_G2Mi2Ao:
     pulse_input_sum2_counter() # Count from 0 to 3 in 1 second steps
     pulse_input_dif2()
     '''
-    def __init__(self):
-        self.node = 1
-        self.name = ''
-        self.id = [2470, 2471, 2472]
-        self.brick_id = ''
-        self.brick_no = ''
+    def __init__(self, node, name, brick_no, brick_id, data_length_miso, data_length_mosi, offset_miso, offset_mosi):
+        self.node = node
+        self.name = name
+        #self.id = [2470, 2471, 2472]
+        self.brick_id = brick_id
+        self.brick_no = brick_no
         # Data Length
-        self.data_length_miso = ''
-        self.data_length_mosi = ''
+        self.data_length_miso = data_length_miso
+        self.data_length_mosi = data_length_mosi
         # Offset
-        self.offset_miso = ''
-        self.offset_mosi = ''
+        self.offset_miso = offset_miso
+        self.offset_mosi = offset_mosi
         ##
         self.mi1_mode = ''
         self.mi2_mode = ''
@@ -488,7 +488,13 @@ class CAE_G2Mi2Ao:
         self.oldPulse2 = 0
         self.pulse_sum1 = 0
         self.pulse_sum2 = 0
+        self.run = False
 
+    def start(self):
+        self.run = True
+    
+    def stop(self):
+        self.run = False
 
     def analog_output1(self, value):
         '''
@@ -651,15 +657,16 @@ class CAE_G2Mi2Ao:
         '''
         Calculate and Return the pulse sum of Multi Input 1
         '''
-        lock.acquire()
-        pulse = ((bB_update.updated[self.node][self.offset_miso + 5] & 0x3F) << 8) + bB_update.updated[self.node][
-            self.offset_miso + 6]
-        impuls_freq = (bB_update.updated[self.node][self.offset_miso + 7] << 8) + bB_update.updated[self.node][
-            self.offset_miso + 8]
-        if self.oldPulse1 != pulse:
-            self.pulse_sum1 += 1 if impuls_freq == 0 else impuls_freq
-            self.oldPulse1 = pulse
-        lock.release()
+        if self.run:
+            lock.acquire()
+            pulse = ((bB_update.updated[self.node][self.offset_miso + 5] & 0x3F) << 8) + bB_update.updated[self.node][
+                self.offset_miso + 6]
+            impuls_freq = (bB_update.updated[self.node][self.offset_miso + 7] << 8) + bB_update.updated[self.node][
+                self.offset_miso + 8]
+            if self.oldPulse1 != pulse:
+                self.pulse_sum1 += 1 if impuls_freq == 0 else impuls_freq
+                self.oldPulse1 = pulse
+            lock.release()
         if self.pulse_sum1 >= sys.maxsize:
             self.pulse_sum1 = 0
         return self.pulse_sum1
@@ -688,15 +695,16 @@ class CAE_G2Mi2Ao:
         '''
         Calculate and Return the pulse sum of Multi Input 2
         '''
-        lock.acquire()
-        pulse = ((bB_update.updated[self.node][self.offset_miso + 9] & 0x3F) << 8) + bB_update.updated[self.node][
-            self.offset_miso + 10]
-        impuls_freq = (bB_update.updated[self.node][self.offset_miso + 11] << 8) + bB_update.updated[self.node][
-            self.offset_miso + 12]
-        if self.oldPulse2 != pulse:
-            self.pulse_sum2 += 1 if impuls_freq == 0 else impuls_freq
-            self.oldPulse2 = pulse
-        lock.release()
+        if self.run:
+            lock.acquire()
+            pulse = ((bB_update.updated[self.node][self.offset_miso + 9] & 0x3F) << 8) + bB_update.updated[self.node][
+                self.offset_miso + 10]
+            impuls_freq = (bB_update.updated[self.node][self.offset_miso + 11] << 8) + bB_update.updated[self.node][
+                self.offset_miso + 12]
+            if self.oldPulse2 != pulse:
+                self.pulse_sum2 += 1 if impuls_freq == 0 else impuls_freq
+                self.oldPulse2 = pulse
+            lock.release()
         if self.pulse_sum2 >= sys.maxsize:
             self.pulse_sum2 = 0
         return self.pulse_sum2
@@ -725,9 +733,6 @@ class CAE_G2Mi2Ao:
         return ai_state
 
 
-G2Mi2Ao = CAE_G2Mi2Ao()
-
-
 class CAE_G4Ai4Tmp:
     '''
     Brick Information can find in eB_products.pdf unter Chapter 1.6.8 G-4Ai4Tmp-0#
@@ -741,19 +746,18 @@ class CAE_G4Ai4Tmp:
     temp_input3()
     temp_input4()
     '''
-
-    def __init__(self):
-        self.node = 1
-        self.name = ''
+    def __init__(self, node, name, brick_no, brick_id, data_length_miso, data_length_mosi, offset_miso, offset_mosi):
+        self.node = node
+        self.name = name
         self.id = [2431, 2432, 2433, 2434, 2435, 2436]
-        self.brick_id = ''
-        self.brick_no = ''
+        self.brick_id = brick_id
+        self.brick_no = brick_no
         # Data Length
-        self.data_length_miso = ''
-        self.data_length_mosi = ''
+        self.data_length_miso = data_length_miso
+        self.data_length_mosi = data_length_mosi
         # Offset
-        self.offset_miso = ''
-        self.offset_mosi = ''
+        self.offset_miso = offset_miso
+        self.offset_mosi = offset_mosi
         self.digit_pt1000 = [0, 64, 128, 192, 256, 320, 384, 448, 512, 576, 640, 704, 768, 832, 896, 1000]
         self.temp_pt1000 = [-78.276, -56.207, -33.929, -10.714, 13.6, 38.8, 65.217, 92.609, 121.364, 151.429, 183.0, 215.789,
                 250.556, 286.667, 325.0, 378.571]
@@ -893,5 +897,3 @@ class CAE_G4Ai4Tmp:
         return ti
 
 
-G4Ai4Tmp = CAE_G4Ai4Tmp()
-
